#include"dottle.hpp"




namespace{


enum{
   left_row_number = 16,
  right_row_number = 18,

};


void
draw_left(Area&  area)
{
    rep(i,left_row_number)
    {
      const int  y = (font::h*i);

      auto  color_index = palette.get_color_index(i);

      const auto&  p = palette.colors[color_index];

      const bool  focused = (i == palette.get_left_index());

      area.fill_rect(0           ,y,(font::w*10),font::h,focused? white_color:black_color);
      area.fill_rect((font::w*10),y,(font::w* 2),font::h,p.base_color);

      area.draw_text(0,y,focused? black_color:white_color,p.name);
    }
}


void
touch_left(Area&  area, const Mouse&  mouse)
{
    if(mouse.left.test_pressed())
    {
      palette.set_left_index(mouse.get_rel_y()/font::h);

      palette.widgets.left_area->request_redraw();
      palette.widgets.light_area->request_redraw();
    }
}




void
draw_right(Area&  area)
{
    rep(i,right_row_number)
    {
      const int  y = (font::h*i);

      auto  color_index = (palette.get_right_index_base()+i);

      const auto&  p = palette.colors[color_index];

      area.fill_rect(           0,y,(font::w*10),font::h, black_color);
      area.fill_rect((font::w*10),y,(font::w* 2),font::h,p.base_color);

      area.draw_text(0,y,white_color,p.name);
    }
}


void
touch_right(Area&  area, const Mouse&  mouse)
{
    if(mouse.left.test_pressed())
    {
      palette.set_right_index_offset(mouse.get_rel_y()/font::h);

      palette.get_color_index() = palette.get_right_index();

      palette.widgets.left_area->request_redraw();
      palette.widgets.light_area->request_redraw();

      view.widgets.update();
    }
}


void
turn_page(Counter&  cntr, int  newval, int  oldval)
{
  palette.set_right_index_base(right_row_number*(newval-1));

  palette.widgets.right_area->request_redraw();
}




void
draw_light(Area&  area)
{
  char  buf[8];

    rep(i,7)
    {
      const int  y = (font::h*i);

      const bool  focused = (i == palette.get_light_index());

      snprintf(buf,sizeof(buf),"%d",i);

      area.fill_rect(0,y,32,font::h,(focused? white_color:black_color));

      area.draw_text(8,y,(focused? black_color:white_color),to_u16string(buf));


      Pixel  pix(i,palette.get_left_index());

      area.fill_rect(32,y,32,font::h,pix.get_lighted_color());
    }
}


void
touch_light(Area&  area, const Mouse&  mouse)
{
    if(mouse.left.test_pressed())
    {
      palette.set_light_index(mouse.get_rel_y()/font::h);

      area.request_redraw();
    }
}


}


void
Palette::Widgets::
initialize()
{
  palette.reset_colors();

   left_area = new Area(font::w*12,font::h* left_row_number);
  right_area = new Area(font::w*12,font::h*right_row_number);
  light_area = new Area(   64,font::h* 7);

   left_area->set_draw_callback(draw_left );
  right_area->set_draw_callback(draw_right);
  light_area->set_draw_callback(draw_light);

   left_area->set_callback(touch_left);
  right_area->set_callback(touch_right);
  light_area->set_callback(touch_light);

  counter = new Counter(1,palette.colors.size()/right_row_number);

  counter->set_callback(turn_page);


  node = new Node;

  int  x = 0;
  int  y = 0;

  node->insert(light_area,x,24);  x += 80;
  node->insert(counter,x+160,y);  y += 24;

  node->insert( left_area,x,y);
  node->insert(right_area,x+160,y);
}




