#ifndef _PAPER_
#define _PAPER_


#include"chunk.hpp"
#include<deque>




enum
UndoType
{
  UDO_string,
  UDO_stroke,

};


enum
CopyType
{
  CPY_nochange,
  CPY_mirror_left,
  CPY_mirror_upper,
  CPY_revolve_left,
  CPY_revolve_right,
  CPY_reverse_vertical,
  CPY_reverse_horizontal,

};


class
Paper
{
public:
  enum{size=320};

  using TwoPointFunction = void  (Paper::*)(const TwoPoints&  points, Pixel  pixel);
  using    FrameFunction = void  (Paper::*)(const TwoPoints&  points              );

private:
  using DotString = std::vector<Dot>;

  std::deque<std::pair<const UndoType,DotString> >  dots_buffer;

  DotString         dots;
  DotString  backup_dots;

  Pixel  pixels[size][size];

public:
  Paper();


  Pixel  get(int  x, int  y) const;

  void  put(int  x, int  y, const Pixel&  pixel);
  void  put(int  x, int  y, const Chunk&  chunk);

  void  put_provisionally(int  x, int  y, const Pixel&  pixel);
  void  put_provisionally(int  x, int  y, const Chunk&  chunk);

  void  restore_backups();
  void    apply_backups();

  Chunk  make_chunk(const SDL_Rect&  rect, CopyType  type=CPY_nochange) const;

  void  clear();


  void  store_string();
  void  store_stroke();
  void  undo();


  void    draw_line(const TwoPoints&  points, Pixel  pixel);
  void    draw_rect(const TwoPoints&  points, Pixel  pixel);
  void  draw_circle(const TwoPoints&  points, Pixel  pixel);


  void  fill_rect(const TwoPoints&  points, Pixel  pixel);

  void  fill_area(int  x, int  y, Pixel  pixel          );
  void  fill_area(int  x, int  y, Pixel      a, Pixel  b);

  void     shift_up(const TwoPoints&  points);
  void   shift_left(const TwoPoints&  points);
  void  shift_right(const TwoPoints&  points);
  void   shift_down(const TwoPoints&  points);

  void   mirror_left(const TwoPoints&  points);
  void  mirror_upper(const TwoPoints&  points);

  void   revolve_left(const TwoPoints&  points);
  void  revolve_right(const TwoPoints&  points);

  void  reverse_h(const TwoPoints&  points);
  void  reverse_v(const TwoPoints&  points);


  bool  load(const char*  path,       Palette&  palette)      ;
  bool  save(const char*  path, const Palette&  palette) const;

  bool  save_as_png(const char*  path, const Palette&  palette) const;

};



#endif




