#include"dottle.hpp"




namespace{


void
react_2p(const Mouse&  mouse, Paper::TwoPointFunction  fn)
{
    if(frame.get_phase() != Frame::PH_second)
    {
      return;
    }


  auto&  points = frame.get_two_points();

    if(mouse.left.test_pressing())
    {
      paper.restore_backups();

      (paper.*fn)(points,palette.make_pixel());
    }

  else
    if(mouse.left.test_unpressed())
    {
      paper.store_string();

      paper.apply_backups();

      paper.store_stroke();

      frame.reset();
    }
}


void
react_sub(const Mouse&  mouse)
{
    switch(get_paint_mode())
    {
      case(PAI_draw_point):
          if(mouse.left.test_pressing())
          {
            paper.put(brush.get_x(),
                      brush.get_y(),palette.make_pixel());
          }
        break;
      case(PAI_draw_line  ): react_2p(mouse,&Paper::draw_line  );break;
      case(PAI_draw_circle): react_2p(mouse,&Paper::draw_circle);break;
      case(PAI_draw_rect  ): react_2p(mouse,&Paper::draw_rect  );break;
      case(PAI_fill_rect  ): react_2p(mouse,&Paper::fill_rect  );break;

      case(PAI_set_frame):
        break;
      case(PAI_fill_area):
          if(mouse.left.test_pressed())
          {
            paper.store_string();

            paper.fill_area(brush.get_x(),
                            brush.get_y(),palette.make_pixel());

            paper.store_stroke();
          }
        break;
      case(PAI_put_copy):
          if(mouse.left.test_pressing())
          {
            paper.store_string();

            paper.apply_backups();

            paper.store_stroke();
          }

        else
          {
            paper.restore_backups();

            paper.put_provisionally(brush.get_x(),
                                    brush.get_y(),get_copy_chunk());
          }
        break;
      default:;
    }
}


void
react(Area&  area, const Mouse&  mouse)
{
    if(mouse.right.test_pressing())
    {
      frame.reset();

      paper.restore_backups();
    }

  else
    {
      const int  x = (mouse.get_rel_x()/view.get_pixel_size());
      const int  y = (mouse.get_rel_y()/view.get_pixel_size());

      brush.set_x_offset(x);
      brush.set_y_offset(y);

      brush.fix_x_offset();
      brush.fix_y_offset();

        if((x < Paper::size) &&
           (y < Paper::size))
        {
            if(mouse.left.test_pressing())
            {
              frame.set(brush.get_x(),brush.get_y());
            }


          react_sub(mouse);
        }
    }


  brush.widgets.update_label();
  frame.widgets.update_label();

  area.request_redraw();
}


void
draw_grid(Area&  area, const int  w, const int  h)
{
    rep(y,h)
    {
      const int  n = (brush.get_y_base()+y);

      const int  yy = (view.get_pixel_size()*y);

           if(!(n&31)){area.draw_hline(0,yy,area.get_parameter().width,get_default_color(15));}
      else if(!(n& 7)){area.draw_hline(0,yy,area.get_parameter().width,get_default_color( 3));}
    }


    rep(x,w)
    {
      const int  n = (brush.get_x_base()+x);

      const int  xx = (view.get_pixel_size()*x);

           if(!(n&31)){area.draw_vline(xx,0,area.get_parameter().height,get_default_color(15));}
      else if(!(n& 7)){area.draw_vline(xx,0,area.get_parameter().height,get_default_color( 3));}
    }
}


void
draw(Area&  area)
{
  area.fill(palette.get_jiscolor_for_fill().base_color);

  const int  pixsz = view.get_pixel_size();

    rep(y,view.get_row_number())
    {
      const int  yy = brush.get_y_base()+y;

        rep(x,view.get_col_number())
        {
          const int  xx = brush.get_x_base()+x;

          auto  pix = paper.get(xx,yy);

          area.fill_rect((pixsz*x),
                         (pixsz*y),
                         pixsz,
                         pixsz,pix.get_lighted_color());
        }
    }


    if(test_grid_using())
    {
      draw_grid(area,view.get_col_number(),view.get_row_number());
    }


    if(view.test_framing() && (frame.get_phase() == Frame::PH_second))
    {
      auto  frame_rect = make_rect(frame.get_two_points());

      int  x = frame_rect.x*view.get_pixel_size()-(brush.get_x_base()*view.get_pixel_size());
      int  y = frame_rect.y*view.get_pixel_size()-(brush.get_y_base()*view.get_pixel_size());
      int  w = frame_rect.w*view.get_pixel_size();
      int  h = frame_rect.h*view.get_pixel_size();

      area.draw_rect_safety(x,y,w,h,white_color,true);
    }


  int  x = (pixsz*brush.get_x_offset());
  int  y = (pixsz*brush.get_y_offset());

    if(pixsz == 1)
    {
      area.draw_rect(x ,y  ,2,2,black_color);
      area.draw_dot(x+1,y+1,    white_color);
    }

  else
    if(pixsz == 2)
    {
      area.draw_rect(x  ,y  ,3,3,black_color);
      area.draw_rect(x+1,y+1,1,1,white_color);
    }

  else
    {
      area.draw_rect(x  ,y  ,pixsz  ,pixsz  ,black_color);
      area.draw_rect(x+1,y+1,pixsz-2,pixsz-2,white_color);
    }
}




void
react_moving(Area&  area, const Mouse&  mouse)
{
    if(mouse.left.test_pressing())
    {
      const int  w = (view.widgets.pad_w/2);
      const int  h = (view.widgets.pad_h/2);

      const int  x = (mouse.get_rel_x()*View::reduce_value)-(w*View::reduce_value);
      const int  y = (mouse.get_rel_y()*View::reduce_value)-(h*View::reduce_value);

      brush.set_x_base(x);
      brush.set_y_base(y);

      brush.fix_x_base();
      brush.fix_y_base();


      brush.widgets.update_label();
      view.widgets.update();
    }
}




void
draw_moving(Area&  area)
{
  area.fill(0);


  int  x = brush.get_x_base()/View::reduce_value;
  int  y = brush.get_y_base()/View::reduce_value;
  int  w = view.widgets.pad_w;
  int  h = view.widgets.pad_h;

  area.fill_rect(x,y,w,h,white_color);
}


void
change_zoom(Counter&  cntr, int  newval, int  opldval)
{
  view.set_zoom_value(newval);
}


}




void
View::Widgets::
initialize()
{
  drawing_area = new Area(size+4,size+4);

  drawing_area->set_flag(WID_ST_hide_cursor_flag);

  drawing_area->set_draw_callback(::draw);
  drawing_area->set_callback(::react);


  moving_area = new Area(size/reduce_value,size/reduce_value);

  moving_area->set_draw_callback(::draw_moving);
  moving_area->set_callback(::react_moving);


  zoom_widget = new TableColumn;

  auto    label = new Label(u"表示倍率");
  auto  counter = new Counter(Counter::T_up_and_down,zoom_min,zoom_max,100);

  counter->set_callback(change_zoom);

  zoom_widget->insert(label);
  zoom_widget->insert(counter);
}




void
View::Widgets::
update()
{
  drawing_area->request_redraw();
   moving_area->request_redraw();
}




