#include"node.hpp"
#include"root.hpp"
#include"context.hpp"




namespace oat{




Node::
Node():
Widget(WID_node),
root(nullptr)
{
}




Root*
Node::
get_root()
{
  return root;
}


void
Node::
set_root(Root*  _root)
{
  root = _root;

    for(auto&  it: children)
    {
        if(*it == WID_node)
        {
          (*(Node*)it).set_root(_root);
        }
    }
}


void
Node::
join_to(Node&  _parent, int  x, int  y)
{
    if(!root)
    {
      Widget::join_to(_parent,x,y);

      set_root(_parent.get_root());
    }
}


void
Node::
cleanup(int  x, int  y)
{
  widget_parameter.top  = (y+widget_parameter.rel_point.y);
  widget_parameter.left = (x+widget_parameter.rel_point.x);

    for(auto&  it: children)
    {
        if(it->test_flag(WID_ST_dirty_flag))
        {
          it->cleanup(widget_parameter.left,widget_parameter.top);

          widget_parameter.right  = std::max(widget_parameter.right ,it->get_parameter().right );
          widget_parameter.bottom = std::max(widget_parameter.bottom,it->get_parameter().bottom);
        }
    }


  widget_parameter.width  = (widget_parameter.right -widget_parameter.left);
  widget_parameter.height = (widget_parameter.bottom-widget_parameter.top );

  reset_flag(WID_ST_dirty_flag);
}


void
Node::
insert(Widget*  wid)
{
  insert(wid,0,0);
}


void
Node::
insert(Widget*  wid, int  x, int  y)
{
    if(wid)
    {
      children.emplace_back(wid);

      wid->join_to(*this,x,y);
    }
}


void
Node::
erase(WidgetID  id)
{
  auto    it = children.begin();
  auto   end = children.end();

    while(it != end)
    {
        if((*it)->widget_id == id)
        {
          children.erase(it);

          break;
        }


      ++it;
    }
}




void
Node::
process(Mouse&  mouse)
{
    for(auto&  it: children)
    {
      auto&  w = *it;

        if(w.test_flag(WID_ST_active_flag) && mouse.test_enter(w))
        {
            if(w != WID_node)
            {
              mouse.reset_rel(w);

              root->reenter(w);
            }


          w.process(mouse);

          return;
        }
    }


  root->reenter(*this);
}




void
Node::
draw()
{
    if(test_flag(WID_ST_visible_flag))
    {
        for(auto&  it: children)
        {
            if(it->test_flag(WID_ST_visible_flag))
            {
              it->draw();
            }
        }
    }
}


void
Node::
redraw()
{
    if(test_flag(WID_ST_redraw_flag))
    {
        if(test_flag(WID_ST_visible_flag))
        {
            for(auto&  it: children)
            {
              it->redraw();
            }
        }


      reset_flag(WID_ST_redraw_flag);
    }
}


void
Node::
destruct()
{
    if(test_flag(WID_ST_destruct_flag))
    {
      auto   it = children.begin();
      auto  end = children.end();

        while(it != end)
        {
            if((*it)->test_flag(WID_ST_destruct_me_flag))
            {
              it = children.erase(it);
            }

          else
            {
              ++it;
            }
        }


      reset_flag(WID_ST_destruct_flag);
    }
}




}




