#ifndef OAT_UTIL_HPP
#define OAT_UTIL_HPP


#include<cstdint>
#include<cstring>
#include<cstdio>
#include<cstdlib>
#include<ctime>
#include<string>
#include<vector>
#include<algorithm>
#include<SDL.h>


#define report  printf("[%8ld] %s %4d %s\n",time(nullptr),__FILE__,__LINE__,__func__)
#define  printd(d)  printf("[printd] %8d\n",(d))

#define  rep(v,n)  for(int  v =     0;  v <  (n);  ++v)
#define rrep(v,n)  for(int  v = (n)-1;  v >=  0 ;  --v)


namespace oat{


extern const int  dirsepch;


struct
Point
{
  uint16_t  x;
  uint16_t  y;

};


struct
TwoPoints
{
  uint16_t  x0;
  uint16_t  y0;
  uint16_t  x1;
  uint16_t  y1;

};


SDL_Rect  make_rect(const TwoPoints&  points);


size_t   u16strlen(const char16_t*  str);

size_t   get_utf8_bytes(const char*  s);
size_t  get_utf8_length(const char*  s);

std::u16string  to_u16string(const std::string&     s);
std::string        to_string(const std::u16string&  u16s);

size_t  push_utf8(std::string  s, char16_t  c);


int  fputc16le(int  c, FILE*  f);
int  fputc32le(int  c, FILE*  f);

int  fgetc16le(FILE*  f);
int  fgetc32le(FILE*  f);


template<typename  T>
void
putc(int  c, T&  vec)
{
  vec.push_back((c   )&0xFF);
}


template<typename  T>
void
putc16le(int  c, T&  vec)
{
  vec.push_back((c   )&0xFF);
  vec.push_back((c>>8)&0xFF);
}


template<typename  T>
void
putc32le(int  c, T&  vec)
{
  vec.push_back((c    )&0xFF);
  vec.push_back((c>> 8)&0xFF);
  vec.push_back((c>>16)&0xFF);
  vec.push_back((c>>24)&0xFF);
}


template<typename  T>
int
getc(T&  it)
{
  return *it++;
}


template<typename  T>
int
getc16le(T&  it)
{
  uint16_t  v  = getc(it)   ;
            v |= getc(it)<<8;

  return(int16_t)(v);
}


template<typename  T>
int
getc32le(T&  it)
{
  uint32_t  v  = getc(it)    ;
            v |= getc(it)<< 8;
            v |= getc(it)<<16;
            v |= getc(it)<<24;

  return(int32_t)(v);
}


void  save_file(const void*  src_data, size_t  src_size);
void  save_gzfile(const void*  src_data, size_t  src_size, const char*  path);

std::string    load_file(const char*  path);
std::string  load_gzfile(const char*  path);


}




#endif




