#include"widget.hpp"
#include"surface.hpp"
#include"context.hpp"
#include"font.hpp"




namespace oat{




void
Widget::
fill()
{
  fill(back_color);
}


void
Widget::
fill(Color  color)
{
  fill_rect(0,0,widget_parameter.width,widget_parameter.height,color);
}


uint8_t*
Widget::
get_ptr(int  x, int  y)
{
  x += widget_parameter.left;
  y += widget_parameter.top ;

  return &context::pixels[(context::pitch*y)+(4*x)];
}


void
Widget::
draw_dot(int  x, int  y, Color  color)
{
  *reinterpret_cast<uint32_t*>(get_ptr(x,y)) = color;
}


void
Widget::
draw_dot_safety(int  x, int  y, Color  color)
{
    if((x >=                      0) &&
       (x <  widget_parameter.width) &&
       (y >=                      0) &&
       (y <  widget_parameter.height))
    {
      draw_dot(x,y,color);
    }
}


int
Widget::
draw_iso8859_1(int  _x, int  _y, Color  color, char  c)
{
  auto  p = font::iso8859_1::get_data(c);

    rep(yy,font::h)
    {
      auto  code = *p++;

        rep(xx,font::small_w)
        {
            if(code&0x80)
            {
              draw_dot(_x+xx,_y+yy,color);
            }


          code <<= 1;
        }
    }


   return font::small_w;
}


int
Widget::
draw_unicode(int  _x, int  _y, Color  color, char16_t  c)
{
  auto  p = font::get_data(c);

    if(p)
    {
        rep(yy,font::h)
        {
          uint16_t  code = *p++;

            rep(xx,font::w)
            {
                if(code&0x8000)
                {
                  draw_dot(_x+xx,_y+yy,color);
                }


              code <<= 1;
            }
        }


      return font::w;
    }


  return 0;
}


void
Widget::
draw_asciiz(int  _x, int  _y, Color  color, const std::string&  u8s)
{
    for(auto  c: u8s)
    {
      _x += draw_iso8859_1(_x,_y,color,c);
    }
}


void
Widget::
draw_asciiz(int  _x, int  _y, Color  color, const std::list<char>&  s)
{
    for(auto  c: s)
    {
      _x += draw_iso8859_1(_x,_y,color,c);
    }
}


void
Widget::
draw_text(int  _x, int  _y, Color  color, const std::string&  u8s)
{
  draw_text(_x,_y,color,to_u16string(u8s));
}


void
Widget::
draw_text(int  _x, int  _y, Color  color, const std::u16string&  u16s)
{
    for(auto  c: u16s)
    {
      _x += ((c <= 0xFF)? draw_iso8859_1(_x,_y,color,c):draw_unicode(_x,_y,color,c));
    }
}


void
Widget::
draw(int  _x, int  _y, const Icon&  icon)
{
    rep(yy,Icon::size){
    rep(xx,Icon::size){
      auto  code = icon.data[yy][xx];

        if(!(code&0x8000))
        {
          draw_dot(_x+xx,_y+yy,default_colors[code]);
        }
    }}
}


void
Widget::
draw_vline(int  x, int  y, int  w, Color  color, bool  brok)
{
    if(!brok)
    {
        while(w--)
        {
          draw_dot(x,y++,color);
        }
    }

  else
    {
      w /= 2;

        while(w--)
        {
          draw_dot(x,y,color);

          y += 2;
        }
    }
}


void
Widget::
draw_hline(int  x, int  y, int  w, Color  color, bool  brok)
{
    if(!brok)
    {
        while(w--)
        {
          draw_dot(x++,y,color);
        }
    }

  else
    {
      w /= 2;

        while(w--)
        {
          draw_dot(x,y,color);

          x += 2;
        }
    }
}




void
Widget::
draw_vline_safety(int  x, int  y, int  w, Color  color, bool  brok)
{
    if((x >=                     0) &&
       (x < widget_parameter.width))
    {
        if(y < 0)
        {
            if(w < -y)
            {
              return;
            }


          w += y    ;
               y = 0;
        }


        if((y+w) >= widget_parameter.height)
        {
          w = (widget_parameter.height-y);
        }


      draw_vline(x,y,w,color,brok);
    }
}


void
Widget::
draw_hline_safety(int  x, int  y, int  w, Color  color, bool  brok)
{
    if((y >=                      0) &&
       (y < widget_parameter.height))
    {
        if(x < 0)
        {
            if(w < -x)
            {
              return;
            }


          w += x    ;
               x = 0;
        }


        if((x+w) >= widget_parameter.width)
        {
          w = (widget_parameter.width-x);
        }


      draw_hline(x,y,w,color,brok);
    }
}




void
Widget::
draw_rect(int  x, int  y, int  w, int  h, Color  color, bool  brok)
{
  draw_vline(x    ,y,h,color,brok);
  draw_vline(x+w-1,y,h,color,brok);

  draw_hline(x,y    ,w,color,brok);
  draw_hline(x,y+h-1,w,color,brok);
}


void
Widget::
fill_rect(int  x, int  y, int  w, int  h, Color  color)
{
  x += widget_parameter.left;
  y += widget_parameter.top ;

  auto  p8 = &context::pixels[(context::pitch*y)+(4*x)];

    rep(yy,h)
    {
      auto  p = (uint32_t*)p8                  ;
                           p8 += context::pitch;

        rep(xx,w)
        {
          *p++ = color;
        }
    }
}




void
Widget::
draw_rect_safety(int  x, int  y, int  w, int  h, Color  color, bool  brok)
{
  draw_vline_safety(x    ,y,h,color,brok);
  draw_vline_safety(x+w-1,y,h,color,brok);

  draw_hline_safety(x,y    ,w,color,brok);
  draw_hline_safety(x,y+h-1,w,color,brok);
}




void
Widget::
draw_button(int  x, int  y, int  w, int  h)
{
  fill_rect(x,y,w,h,gray_color);

  draw_rect(x,y  ,w,  2,white_color);
  draw_rect(x,y+2,2,h-2,white_color);

  draw_rect(x    ,y+h-2,w,  2,dark_gray_color);
  draw_rect(x+w-2,y+2  ,2,h-2,dark_gray_color);
}


void
Widget::
draw_sank_button(int  x, int  y, int  w, int  h)
{
  fill_rect(x,y,w,h,dark_gray_color);

  draw_rect(x  ,y  ,w  ,h  ,dark_gray_color);
  draw_rect(x+1,y+1,w-2,h-2,dark_gray_color);

  draw_vline(x+w-1,y+  1,h-2,white_color);
  draw_hline(x+  1,y+h-1,w-2,white_color);
}
 

void
Widget::
draw_input_form(int  x, int  y, int  w, int  h)
{
  fill_rect(x+1,y+1,w-2,h-2,white_color);

  draw_hline(0  ,0  ,w,black_color);
  draw_vline(0  ,0  ,h,black_color);
  draw_hline(0  ,h-1,w,white_color);
  draw_vline(w-1,0  ,h,white_color);
}




}




